#include <stdio.h>
#include "RemoteTable.h"
#include <QStringList>
#include <QHeaderView>
#include "OneButton.h"

RemoteTable::RemoteTable()
 : QTableWidget(0, 3),
   appending_row(false),
   changing_row(false),
   in_do_item_changed(false),
   red_question(":/new/prefix1/images/red-question.png"),
   green_question(":/new/prefix1/images/green-question.png"),
   green_check(":/new/prefix1/images/green-check.png") {

   QStringList headers;
   headers << "Button" << "Status" << "Info";
   setHorizontalHeaderLabels(headers);
   horizontalHeader()->setStretchLastSection(true);
   verticalHeader()->hide();

   connect(this, SIGNAL(itemChanged(QTableWidgetItem *)), this,
           SLOT(doItemChanged(QTableWidgetItem *)));

   appendNewRow();
}

RemoteTable::~RemoteTable() {
   return;
}

void
RemoteTable::SetChangingRow(bool change_status) {
   changing_row = change_status;
}

void
RemoteTable::CheckLastRow() {
   if (rowCount() == 0) {
      appendNewRow();
   }
}

void
RemoteTable::CopyData() {
   int rows = rowCount();
   for (int r = 0; r < rows; ++r) {
      OneButton * obp = dynamic_cast<OneButton *>(item(r, 0));
      if (obp != NULL) {
         obp->CopyData();
      }
   }
}

void
RemoteTable::RestoreCopy() {
   int rows = rowCount();
   for (int r = 0; r < rows; ++r) {
      OneButton * obp = dynamic_cast<OneButton *>(item(r, 0));
      if (obp != NULL) {
         obp->RestoreCopy();
      }
   }
}

void
RemoteTable::doItemChanged(QTableWidgetItem * item) {
   if (in_do_item_changed ||
       appending_row ||
       (column(item) != 0) ||
       (item->text().isEmpty())) {
      return;
   }
   in_do_item_changed = true;

   // Make sure we have a legal button name, if not, fix it up.
   // (Legal -> no double quote chars, no duplicates).

   QString old_name = item->text();
   QString new_name = old_name;
   new_name.remove('\"');
   QString base_name = new_name;
   int unique = 2;
   for ( ; ; ) {
      int dups = CountDuplicates(new_name, item);
      if (dups > 0) {
         new_name = base_name;
         new_name.append(QString("<%1>").arg(unique++));
      } else {
         break;
      }
   }
   if (old_name != new_name) {
      item->setText(new_name);
   }

   if (row(item) == (rowCount() - 1)) {
      appendNewRow();
   }
   in_do_item_changed = false;
}

void
RemoteTable::appendNewRow() {
   appending_row = true;
   int rows = rowCount();
   insertRow(rows);
   OneButton * i = dynamic_cast<OneButton *>(item(rows, 0));
   if (i == NULL) {
      i = new OneButton;
      setItem(rows, 0, i);
      i->UpdateButtonInfo();
   }
   if (! changing_row) {
      selectRow(rows);
   }
   appending_row = false;
}

int
RemoteTable::CountDuplicates(QString txt, QTableWidgetItem * skip_item) {
   int dups = 0;
   int rows = rowCount();
   int skip_row = row(skip_item);
   for (int r = 0; r < rows; ++r) {
      if (r != skip_row) {
         QTableWidgetItem * itp = item(r, 0);
         if ((itp != NULL) && (txt == itp->text())) {
            ++dups;
         }
      }
   }
   return dups;
}
